from dataclasses import dataclass

from purgatory.typing import TTL, Threshold

from .base import Command


@dataclass(frozen=True)
class CreateCircuitBreaker(Command):
    name: str
    threshold: Threshold
    ttl: TTL



from typing import Any

try:
    from redis.asyncio import Redis as AioRedis

    AsyncRedis = AioRedis
except ImportError:
    AsyncRedis = Any  # type: ignore

try:
    from redis import Redis

    SyncRedis = Redis
except ImportError:
    SyncRedis = Any  # type: ignore



"""
Message base classes.

`Command` and `Event` are two types used to handle changes in the model.

"""


class Message:
    """Base class for messaging."""


class Command(Message):
    """Baseclass for message of type command."""


class Event(Message):
    """Baseclass for message of type event."""



"""
Propagate commands and events to every registered handles.

"""
import logging
from typing import Any, Callable, Coroutine, TypeVar, Union

from purgatory.domain.messages.base import Command, Event

from ._async.unit_of_work import AsyncAbstractUnitOfWork
from ._sync.unit_of_work import SyncAbstractUnitOfWork

log = logging.getLogger(__name__)

TCommand = TypeVar("TCommand", bound=Command)
TEvent = TypeVar("TEvent", bound=Event)

AsyncCommandHandler = Callable[
    [TCommand, AsyncAbstractUnitOfWork], Coroutine[Any, Any, Any]
]
AsyncEventHandler = Callable[
    [TEvent, AsyncAbstractUnitOfWork], Coroutine[Any, Any, None]
]
AsyncMessageHandler = Union[AsyncCommandHandler[TCommand], AsyncEventHandler[TEvent]]


SyncCommandHandler = Callable[[TCommand, SyncAbstractUnitOfWork], Any]
SyncEventHandler = Callable[[TEvent, SyncAbstractUnitOfWork], None]
SyncMessageHandler = Union[SyncCommandHandler[TCommand], SyncEventHandler[TEvent]]
